<?php

namespace Drupal\nft_oembed\Normalizer;

use Drupal\Core\File\FileSystemInterface;
use Drupal\media\OEmbed\Resource;
use Drupal\serialization\Normalizer\ContentEntityNormalizer;

/**
 * Converts NFT data to OEmbed-compliant array.
 */
class NftNormalizer extends ContentEntityNormalizer {

  protected $logger;
  protected $httpClient;
  protected $fileSystem;
  protected $mimeGuesser;
  protected $fileUrlGenerator;

  public function __construct(
    $entity_type_manager,
    $entity_type_repository,
    $entity_field_manager,
    // Custom additions.
    $logger, $http_client, $file_system, $mime_type_guesser, $file_url_generator
  ) {
    parent::__construct($entity_type_manager, $entity_type_repository, $entity_field_manager);

    $this->logger = $logger;
    $this->httpClient = $http_client;
    $this->fileSystem = $file_system;
    $this->mimeGuesser = $mime_type_guesser;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, $format = NULL) {
    if (!is_object($data) || !$this->checkFormat($format)) {
      return FALSE;
    }

    switch ($data->__get('provider_id')) {
      default:
        // @todo Call custom code per provider.
        break;
    }

    // Otherwise, this normalizer does not support the $data object.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsDenormalization($data, $type, $format = NULL, array $context = []) {
    if (!$this->checkFormat($format)) {
      return FALSE;
    }

    if (
      ($context['nft_oembed'] ?? NULL) &&
      \is_string($context['provider_id'] ?? NULL) &&
      !empty($context['provider_id'])
    ) {
      return TRUE;
    }

    // Otherwise, this normalizer does not support the $data object.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($entity, $format = NULL, array $context = []) {
    $normalized_values = [];

    return $normalized_values;
  }

  /**
   * {@inheritDoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []) {
    $values = [];

    // Required hardcoded version.
    // @see \Drupal\media\OEmbed\ResourceFetcher::createResource();
    $values['version'] = '1.0';

    switch ($context['provider_id'] ?? NULL) {
      case 'Localhost':
        // @todo Custom code per provider.
        break;
      case 'Etherscan':
      case 'OpenSea':
        $values['type'] = Resource::TYPE_PHOTO;
        $values['url'] = $data['image_url'];
        $values['title'] = $data['name'];
        $values['author_name'] = $data['creator']['user']['username'];
        $values['author_url'] = $data['asset_contract']['external_link'];
        $values['thumbnail_url'] = $this->getMediaUrl($data['image_thumbnail_url']);
        $values['thumbnail_width'] = 100;
        $values['thumbnail_height'] = 100;
        break;
      case 'Rarible':
        $meta = $data['meta'];
        $content = $meta['content'];

        $values['type'] = Resource::TYPE_PHOTO;
        $values['title'] = $meta['name'];
        $values['description'] = $meta['description'];

        foreach ($content as $info) {
          // @todo Make acceptable image size editable in config.
          // $acceptable_image_size = 'ORIGINAL';
          $acceptable_image_size = 'BIG';
          switch ($info['representation']) {
            case $acceptable_image_size:
              $values['url'] = $this->getMediaUrl($info['url']);
              break;
            case 'PREVIEW':
              $values['thumbnail_url'] = $info['url'];
              $values['thumbnail_width'] = $info['width'];
              $values['thumbnail_height'] = $info['height'];
              break;
          }
        }

        break;
    }

    return $values;
  }

  /**
   * Handle mixed path to media.
   *
   * Helpful for images from IPFS (e.g. Rarible 'ORIGINAL').
   *
   * @param string $path
   *   A given path to a something online.
   *
   * @return string
   *   A Url.
   */
  protected function getMediaUrl(string $remote_url) {
    // Skip if filename looks okay.
    $mimetype = $this->mimeGuesser->guessMimeType($remote_url);
    if (strpos($mimetype, 'application/octet-stream') !== 0) {
      return $remote_url;
    }

    // Retrieve remote media locally.
    try {
      $parsed_url = parse_url($remote_url);
      $parsed_parts = explode('/', $parsed_url['path']);
      $filename = end($parsed_parts);

      $response = $this->httpClient->get($remote_url);
      $mimetype = $response->getHeader('Content-Type')[0];
      $parsed_mimetype = explode('/', $mimetype);
      $extension = end($parsed_mimetype);
      $data = (string) $response->getBody();

      $directory = 'public://' . 'oembed_nft_media';
      $this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);

      $replace = FileSystemInterface::EXISTS_REPLACE;
      $path = $directory . '/' . md5($filename) . '.' . $extension;
      $this->fileSystem->saveData($data, $path, $replace);
      return $this->fileUrlGenerator->generateAbsoluteString($path);
    } catch (\Exception$e) {
      $this->logger->error($e->getMessage());
    }

    return NULL;
  }

}
